/* -*-c-*-
 *
 * $Id: buf.c,v 1.1 2001/02/03 20:26:37 mdw Exp $
 *
 * Buffer handling
 *
 * (c) 2001 Straylight/Edgeware
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * This file is part of Trivial IP Encryption (TrIPE).
 *
 * TrIPE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * TrIPE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with TrIPE; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*----- Revision history --------------------------------------------------* 
 *
 * $Log: buf.c,v $
 * Revision 1.1  2001/02/03 20:26:37  mdw
 * Initial checkin.
 *
 */

/*----- Header files ------------------------------------------------------*/

#include "tripe.h"

/*----- Main code ---------------------------------------------------------*/

/* --- @buf_init@ --- *
 *
 * Arguments:	@buf *b@ = pointer to a buffer block
 *		@void *p@ = pointer to a buffer
 *		@size_t sz@ = size of the buffer
 *
 * Returns:	---
 *
 * Use:		Initializes the buffer block appropriately.
 */

void buf_init(buf *b, void *p, size_t sz)
{
  b->base = b->p = p;
  b->limit = b->p + sz;
  b->f = 0;
}

/* --- @buf_break@ --- *
 *
 * Arguments:	@buf *b@ = pointer to a buffer block
 *
 * Returns:	Some negative value.
 *
 * Use:		Marks a buffer as broken.
 */

int buf_break(buf *b) { b->f |= BF_BROKEN; return (-1); }

/* --- @buf_ensure@ --- *
 *
 * Arguments:	@buf *b@ = pointer to a buffer block
 *		@size_t sz@ = size of data wanted
 *
 * Returns:	Zero if it worked, nonzero if there wasn't enough space.
 *
 * Use:		Ensures that there are @sz@ bytes still in the buffer.
 */

int buf_ensure(buf *b, size_t sz) { return (BENSURE(b, sz)); }

/* --- @buf_get@ --- *
 *
 * Arguments:	@buf *b@ = pointer to a buffer block
 *		@void *p@ = pointer to a buffer
 *		@size_t sz@ = size of the buffer
 *
 * Returns:	Zero if it worked, nonzero if there wasn't enough data.
 *
 * Use:		Fetches data from the buffer into some other place.
 */

int buf_get(buf *b, void *p, size_t sz)
{
  if (BENSURE(b, sz))
    return (-1);
  memcpy(p, BCUR(b), sz);
  BSTEP(b, sz);
  return (0);
}

/* --- @buf_put@ --- *
 *
 * Arguments:	@buf *b@ = pointer to a buffer block
 *		@const void *p@ = pointer to a buffer
 *		@size_t sz@ = size of the buffer
 *
 * Returns:	Zero if it worked, nonzero if there wasn't enough space.
 *
 * Use:		Fetches data from some place and puts it in the buffer
 */

int buf_put(buf *b, const void *p, size_t sz)
{
  if (BENSURE(b, sz))
    return (-1);
  memcpy(BCUR(b), p, sz);
  BSTEP(b, sz);
  return (0);
}

/* --- @buf_getbyte@ --- *
 *
 * Arguments:	@buf *b@ = pointer to a buffer block
 *
 * Returns:	A byte, or less than zero if there wasn't a byte there.
 *
 * Use:		Gets a single byte from a buffer.
 */

int buf_getbyte(buf *b)
{
  if (BENSURE(b, 1))
    return (-1);
  return (*b->p++);
}

/* --- @buf_putbyte@ --- *
 *
 * Arguments:	@buf *b@ = pointer to a buffer block
 *		@int ch@ = byte to write
 *
 * Returns:	Zero if OK, nonzero if there wasn't enough space.
 *
 * Use:		Puts a single byte in a buffer.
 */

int buf_putbyte(buf *b, int ch)
{
  if (BENSURE(b, 1))
    return (-1);
  *b->p++ = ch;
  return (0);
}

/* --- @buf_getword@ --- *
 *
 * Arguments:	@buf *b@ = pointer to a buffer block
 *		@uint32 *w@ = where to put the word
 *
 * Returns:	Zero if OK, or nonzero if there wasn't a word there.
 *
 * Use:		Gets a 32-bit word from a buffer.
 */

int buf_getword(buf *b, uint32 *w)
{
  if (BENSURE(b, 4))
    return (-1);
  *w = LOAD32(b->p);
  BSTEP(b, 4);
  return (0);
}

/* --- @buf_putword@ --- *
 *
 * Arguments:	@buf *b@ = pointer to a buffer block
 *		@uint32 w@ = word to write
 *
 * Returns:	Zero if OK, nonzero if there wasn't enough space.
 *
 * Use:		Puts a 32-but word in a buffer.
 */

int buf_putword(buf *b, uint32 w)
{
  if (BENSURE(b, 4))
    return (-1);
  STORE32(b->p, w);
  BSTEP(b, 4);
  return (0);
}

/* --- @buf_getmp@ --- *
 *
 * Arguments:	@buf *b@ = pointer to a buffer block
 *
 * Returns:	A multiprecision integer, or null if there wasn't one there.
 *
 * Use:		Gets a multiprecision integer from a buffer.
 */

mp *buf_getmp(buf *b, mp *d)
{
  uint32 sz;
  if (buf_getword(b, &sz) || buf_ensure(b, sz))
    return (0);
  d = mp_loadb(d, BCUR(b), sz);
  BSTEP(b, sz);
  return (d);
}

/* --- @buf_putmp@ --- *
 *
 * Arguments:	@buf *b@ = pointer to a buffer block
 *		@mp *m@ = a multiprecision integer
 *
 * Returns:	Zero if it worked, nonzero if there wasn't enough space.
 *
 * Use:		Puts a multiprecision integer to a buffer.
 */

int buf_putmp(buf *b, mp *m)
{
  size_t sz = mp_octets(m);
  if (buf_putword(b, sz) || buf_ensure(b, sz))
    return (-1);
  mp_storeb(m, BCUR(b), sz);
  BSTEP(b, sz);
  return (0);
}

/*----- That's all, folks -------------------------------------------------*/
